{% load static %}

<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <meta http-equiv="X-UA-Compatible" content="ie=edge">
  <title>{% block title %}{% endblock %}</title>
</head>
<body>
<main>
  {% block main %}{% endblock %}
</main>
<script src="https://cdn.jsdelivr.net/npm/p5@1.9.0/lib/p5.js"></script>
<script src="{% static 'cardroom/felt.js' %}"></script>
{% block scripts %}{% endblock %}
</body>
</html>
